/**
 * Copyright (C) 2001-2002
 * - France Telecom R&D
 * - Laboratoire Logiciels, Systemes, Reseaux - UMR 5526, CNRS-INPG-UJF
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors:
 *
 */

package org.objectweb.perseus.concurrency.api;

/**
 * This is the interface for Concurrency Control service.
 *
 * @author Luciano Garcia-Banuelos (Luciano.Garcia@imag.fr)
 */
public interface ConcurrencyManager {

    /**
	 * This method records the start of an execution context. It can be a
	 * transaction starting.
	 * @param ctx is the context
	 */
	void begin(Object ctx);

    /**
	 * This method requests the validation os accesses made by the context. It
	 * retrieves a boolean value to indicate if the accesses are validate or
	 * not.
	 */
	boolean validate(Object ctx);

    /**
	 * This method marks the end of accesses made by the execution context. This
	 * method should be called when the validate method has returned true.
	 */
	void finalize(Object ctx);

	/**
	 * This method allows to release the resources allocated in a given context.
	 * This method should be called when the validate method has returned false.
	 */
    void abort(Object ctx);

	/**
	 * This method records an access intention to a data object in read mode.
	 * @param resourceId is the resource identifier
	 * @param ctx is an identifier of the execution context. It can be a
	 * transaction handle.
	 * @param hints is an object parameter permiting to some implementation to
     * do some action at "readIntention time".
     * @return null or an object depending on the ConcurrencyManager type
	 * @exception ConcurrencyException
	 */
    Object readIntention(Object ctx, Object resourceId, Object hints)
			throws ConcurrencyException;

	/**
	 * This method records an access intention to a data object in write mode.
	 * A call to the readIntention is necessary before a call to this method.
     * @param resourceId is the resource identifier
	 * @param ctx is an identifier of the execution context. It can be a
	 * transaction handle.
     * @param hints is an object parameter permiting to some implementation to
     * do some action at "readIntention time".
     * @return null or an object depending on the ConcurrencyManager type
	 * @exception ConcurrencyException
	 */
    Object writeIntention(Object ctx, Object resourceId, Object hints)
			throws ConcurrencyException;
}
