package org.objectweb.perseus.concurrency.lib;


/**
 * Copyright (C) 2003-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors: Olivier Lobry (olivier.lobry@rd.francetelecom.com)
 *
 */
public class ReadWriteLockValue implements LockValue {

	public static final byte READ = 1;
	public static final byte UPGRADE = 2;
	public static final byte WRITE = 3;


/*
 *       3  2  1  0
 *       X  U  S  N
 * 0: N  1  1  1  1  =    15
 * 1: S  0  0  1  1  =    3
 * 2: U  0  0  0  1  =    1
 * 3: X  0  0  0  1  =    1
 */

	static final byte compTable[] = { 15, 3, 1, 1 };


	public byte maxValue() {
		return WRITE;
	}

	public boolean isCompatibleWith(byte l1, byte l2) {
		boolean res = (((1 << l1) & compTable[l2]) != 0);
		return res;
	}

	public byte getCompatibleWith(byte l1, byte l2) {
		while (!isCompatibleWith(l1, l2)) l1--;
		return l1;
	}

}
